package com.cardiff.sytest;

import com.cardiff.client_project.controller.nurse.NurseController;
import com.cardiff.client_project.pojo.entity.Nurse;
import com.cardiff.client_project.pojo.vo.NurseVO;
import com.cardiff.client_project.service.NurseService;
import com.cardiff.client_project.utils.Result;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import com.fasterxml.jackson.databind.ObjectMapper;
import java.util.Arrays;
import java.util.List;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

@WebMvcTest(NurseController.class)
@Import(NurseTestSecurityConfig.class)
public class LightweightMockMVCTests {

    @Autowired
    private MockMvc mockMvc;

    @MockBean
    private NurseService nurseService;

    @Autowired
    private ObjectMapper objectMapper;

    // 测试查询所有护士
    @Test
    @WithMockUser(roles = "HOSPITAL")
    void testSelectAll() throws Exception {
        NurseVO nurseVO = new NurseVO();
        nurseVO.setName("Test Nurse");
        when(nurseService.selectAllNurse(anyInt()))
                .thenReturn(Result.success(Arrays.asList(nurseVO)));

        mockMvc.perform(post("/nurse/selectAll")
                        .param("hospitalId", "1"))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.data[0].name").value("Test Nurse"));
    }

    // 测试条件查询
    @Test
    @WithMockUser(roles = "HOSPITAL")
    void testSelectByItem() throws Exception {
        Nurse nurse = new Nurse();
        nurse.setName("Test");

        NurseVO nurseVO = new NurseVO();
        nurseVO.setName("Test Nurse");

        when(nurseService.selectByItem(any(Nurse.class)))
                .thenReturn(Result.success(Arrays.asList(nurseVO)));

        mockMvc.perform(post("/nurse/select")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(nurse)))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.data[0].name").value("Test Nurse"));
    }

    // 测试更新护士信息
    @Test
    @WithMockUser(roles = "HOSPITAL")
    void testUpdateById() throws Exception {
        Nurse nurse = new Nurse();
        nurse.setId(1);
        nurse.setName("Updated Name");

        when(nurseService.update(any(Nurse.class)))
                .thenReturn(Result.success());

        mockMvc.perform(put("/nurse/update")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(nurse)))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.code").value(1));
    }

    // 测试添加护士
    @Test
    @WithMockUser(roles = "HOSPITAL")
    void testInsertNurse() throws Exception {
        Nurse nurse = new Nurse();
        nurse.setName("New Nurse");
        nurse.setEmail("new@hospital.com");

        when(nurseService.insertNurse(any(Nurse.class)))
                .thenReturn(Result.success());

        mockMvc.perform(post("/nurse/insert")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(nurse)))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.code").value(1));
    }

    // 测试批量删除
    @Test
    @WithMockUser(roles = "HOSPITAL")
    void testDeleteNurseById() throws Exception {
        List<Integer> ids = Arrays.asList(1, 2, 3);

        when(nurseService.deleteById(eq(ids), eq("hospital")))
                .thenReturn(Result.success());

        mockMvc.perform(delete("/nurse/deleteNurseById")
                        .contentType(MediaType.APPLICATION_JSON)
                        .content(objectMapper.writeValueAsString(ids)))
                .andExpect(status().isOk());
    }
}